import { factorySpace } from "../node_modules/.pnpm/micromark-factory-space@2.0.1/node_modules/micromark-factory-space/index.mjs";
import { markdownLineEnding } from "../node_modules/.pnpm/micromark-util-character@2.1.1/node_modules/micromark-util-character/index.mjs";
import { ThinkEvent } from "./type.mjs";
import { types } from "../node_modules/.pnpm/micromark-util-symbol@2.0.1/node_modules/micromark-util-symbol/lib/types.mjs";
import { codes } from "../node_modules/.pnpm/micromark-util-symbol@2.0.1/node_modules/micromark-util-symbol/lib/codes.mjs";
import { values } from "../node_modules/.pnpm/micromark-util-symbol@2.0.1/node_modules/micromark-util-symbol/lib/values.mjs";
import { constants } from "../node_modules/.pnpm/micromark-util-symbol@2.0.1/node_modules/micromark-util-symbol/lib/constants.mjs";
const getStrCode = (str) => {
  const key = Object.keys(values).find((key2) => values[key2] === str);
  return codes[key];
};
const thinkFlow = (options) => {
  const { customTagName = "think" } = options || {};
  if (typeof customTagName !== "string") {
    throw new Error("customTagName must be a string");
  }
  return {
    tokenize: tokenizeThinkFenced,
    concrete: true,
    name: "thinkFlow"
  };
  function tokenizeThinkFenced(effects, ok, nok) {
    const self = this;
    const tail = self.events[self.events.length - 1];
    const initialSize = tail && tail[1].type === types.linePrefix ? tail[2].sliceSerialize(tail[1], true).length : 0;
    let thinkState = 0;
    const codeTags = Array.from(customTagName).map((i) => getStrCode(i));
    codeTags.push(codes.greaterThan);
    return start;
    function start(code) {
      if (code !== codes.lessThan) return nok(code);
      effects.enter(ThinkEvent.thinkFlow);
      effects.enter(ThinkEvent.thinkFlowFence);
      effects.consume(code);
      effects.enter(ThinkEvent.thinkFlowFenceSequence);
      return openTag;
    }
    function openTag(code) {
      const strLength = codeTags.length - 1;
      const needCode = codeTags[thinkState];
      if (code !== needCode) return nok(code);
      if (thinkState < strLength) {
        effects.consume(code);
        thinkState++;
        return openTag;
      }
      if (thinkState === codeTags.length - 1 && needCode === codes.greaterThan) {
        effects.consume(code);
        effects.exit(ThinkEvent.thinkFlowFenceSequence);
        return factorySpace(effects, afterOpenSequence, types.whitespace);
      }
      return nok(code);
    }
    function afterOpenSequence(code) {
      if (code === codes.eof || markdownLineEnding(code)) {
        effects.exit(ThinkEvent.thinkFlowFence);
        if (self.interrupt) {
          return ok(code);
        }
        return effects.attempt(nonLazyContinuation, beforeContent, after)(code);
      }
      return nok(code);
    }
    function beforeContent(code) {
      return effects.attempt(
        { tokenize: tokenizeClosingFence, partial: true },
        after,
        contentStart
      )(code);
    }
    function contentStart(code) {
      return (initialSize ? factorySpace(
        effects,
        beforeContentChunk,
        types.linePrefix,
        initialSize + 1
      ) : beforeContentChunk)(code);
    }
    function beforeContentChunk(code) {
      if (code === codes.eof) {
        return after(code);
      }
      if (markdownLineEnding(code)) {
        return effects.attempt(nonLazyContinuation, beforeContent, after)(code);
      }
      effects.enter(ThinkEvent["thinkFlowValue"]);
      return contentChunk(code);
    }
    function contentChunk(code) {
      if (code === codes.eof || markdownLineEnding(code)) {
        effects.exit(ThinkEvent["thinkFlowValue"]);
        return beforeContentChunk(code);
      }
      effects.consume(code);
      return contentChunk;
    }
    function after(code) {
      effects.exit(ThinkEvent.thinkFlow);
      return ok(code);
    }
    function tokenizeClosingFence(effects2, ok2, nok2) {
      let matched = false;
      return factorySpace(
        effects2,
        closingPrefixAfter,
        types.linePrefix,
        self.parser.constructs.disable.null.includes("codeIndented") ? void 0 : constants.tabSize
      );
      function closingPrefixAfter(code) {
        if (code === codes.lessThan) {
          effects2.enter(ThinkEvent.thinkFlowFence);
          effects2.enter(ThinkEvent.thinkFlowFenceSequence);
          effects2.enter(ThinkEvent.thinkFlowCloseFenceSequence);
          effects2.consume(code);
          return closingSlash;
        }
        return nok2(code);
      }
      function closingSlash(code) {
        if (code === codes.slash) {
          thinkState = 0;
          effects2.consume(code);
          return closingTag;
        }
        return nok2(code);
      }
      function closingTag(code) {
        const strLength = codeTags.length - 1;
        const needCode = codeTags[thinkState];
        if (code !== needCode) return nok2(code);
        if (thinkState < strLength) {
          effects2.consume(code);
          thinkState++;
          return closingTag;
        }
        if (thinkState === codeTags.length - 1 && needCode === codes.greaterThan) {
          matched = true;
          return closingTagEnd(code);
        }
        return nok2(code);
      }
      function closingTagEnd(code) {
        if (code === codes.greaterThan && matched) {
          effects2.consume(code);
          effects2.exit(ThinkEvent.thinkFlowCloseFenceSequence);
          effects2.exit(ThinkEvent.thinkFlowFenceSequence);
          return factorySpace(effects2, closingSequenceEnd, types.whitespace);
        }
        return nok2(code);
      }
      function closingSequenceEnd(code) {
        if (code === codes.eof || markdownLineEnding(code)) {
          effects2.exit(ThinkEvent.thinkFlowFence);
          return ok2(code);
        }
        return nok2(code);
      }
    }
  }
};
const nonLazyContinuation = {
  tokenize: tokenizeNonLazyContinuation,
  partial: true
};
function tokenizeNonLazyContinuation(effects, ok, nok) {
  const self = this;
  return start;
  function start(code) {
    if (code === null) {
      return ok(code);
    }
    console.log("expected eol");
    effects.enter(ThinkEvent.lineEnding);
    effects.consume(code);
    effects.exit(ThinkEvent.lineEnding);
    return lineStart;
  }
  function lineStart(code) {
    return self.parser.lazy[self.now().line] ? nok(code) : ok(code);
  }
}
export {
  thinkFlow
};
